#!/usr/bin/env python3.7
# -*- coding: utf-8 -*-

"""
Lire des BME280 en I2C
correction de température par CORtemp()
CDBI	12 Fevrier 2020	Novembre 2021
"""

from collections import namedtuple
import math
import smbus

Data = namedtuple('Data', ['humidity', 'pressure', 'temperature'])

class BME280():

	def __init__(self,Ad):
		self.bus = smbus.SMBus(1)
		self.Ad=Ad
		self.sea_level_pressure=1013.25
		self.cor_t=0.0
		self.t_fine=0.0
		self.setup()
		#Pour vérifier l'accès
		self.read_adc()

	def reset_calibration(self):
		self.calibration_h = []
		self.calibration_p = []
		self.calibration_t = []
		self.t_fine=0.0

	def populate_calibration_data(self):
		self.raw_data = []
		for i in range(0x88, 0x88 + 24):
			self.raw_data.append(self.bus.read_byte_data(self.Ad,i))
		self.raw_data.append(self.bus.read_byte_data(self.Ad,0xA1))
		for i in range(0xE1, 0xE1 + 7):
			self.raw_data.append(self.bus.read_byte_data(self.Ad,i))

		self.calibration_t.append((self.raw_data[1] << 8) | self.raw_data[0])
		self.calibration_t.append((self.raw_data[3] << 8) | self.raw_data[2])
		self.calibration_t.append((self.raw_data[5] << 8) | self.raw_data[4])
		self.calibration_p.append((self.raw_data[7] << 8) | self.raw_data[6])
		self.calibration_p.append((self.raw_data[9] << 8) | self.raw_data[8])
		self.calibration_p.append((self.raw_data[11] << 8) | self.raw_data[10])
		self.calibration_p.append((self.raw_data[13] << 8) | self.raw_data[12])
		self.calibration_p.append((self.raw_data[15] << 8) | self.raw_data[14])
		self.calibration_p.append((self.raw_data[17] << 8) | self.raw_data[16])
		self.calibration_p.append((self.raw_data[19] << 8) | self.raw_data[18])
		self.calibration_p.append((self.raw_data[21] << 8) | self.raw_data[20])
		self.calibration_p.append((self.raw_data[23] << 8) | self.raw_data[22])
		self.calibration_h.append(self.raw_data[24])
		self.calibration_h.append((self.raw_data[26] << 8) | self.raw_data[25])
		self.calibration_h.append(self.raw_data[27])
		self.calibration_h.append((self.raw_data[28] << 4) | (0x0F & self.raw_data[29]))
		self.calibration_h.append((self.raw_data[30] << 4) | ((self.raw_data[29] >> 4) & 0x0F))
		self.calibration_h.append(self.raw_data[31])

		for i in range(1, 2):
			if self.calibration_t[i] & 0x8000:
				self.calibration_t[i] = (-self.calibration_t[i] ^ 0xFFFF) + 1

		for i in range(1, 8):
			if self.calibration_p[i] & 0x8000:
				self.calibration_p[i] = (-self.calibration_p[i] ^ 0xFFFF) + 1

		for i in range(0, 6):
			if self.calibration_h[i] & 0x8000:
				self.calibration_h[i] = (-self.calibration_h[i] ^ 0xFFFF) + 1

	def read_adc(self):
		data = []
		for i in range(0xF7, 0xF7 + 8):
			data.append(self.bus.read_byte_data(self.Ad,i))
		pres_raw = (data[0] << 12) | (data[1] << 4) | (data[2] >> 4)
		temp_raw = (data[3] << 12) | (data[4] << 4) | (data[5] >> 4)
		hum_raw = (data[6] << 8) | data[7]
		return Data(hum_raw, pres_raw, temp_raw)

	def read_all(self):
		try:
			data = self.read_adc()
			return Data(
				self.read_humidity(data),
				self.read_pressure(data),
				self.read_temperature(data))
		except:
			return False

	def read_humidity(self,data=None):
		if data is None:
			data = read_adc(self)
		# Besoin temperature pour calcul humidité
		self.read_temperature(data)
		return self.compensate_humidity(data.humidity)

	def read_pressure(self,data=None):
		#retourne hpa,mm
		if data is None:
			data = self.read_adc()
		# Besoin temperature pour calcul pression
		self.read_temperature(data)
		p=self.compensate_pressure(data.pressure)
		return (p,p/13.332287415)

	def read_temperature(self,data=None):
		if data is None:
			data = self.read_adc()
		return self.compensate_temperature(data.temperature)+self.cor_t

	def compensate_pressure(self,adc_p):
		v1 = (self.t_fine / 2.0) - 64000.0
		v2 = (((v1 / 4.0) * (v1 / 4.0)) / 2048) * self.calibration_p[5]
		v2 += ((v1 * self.calibration_p[4]) * 2.0)
		v2 = (v2 / 4.0) + (self.calibration_p[3] * 65536.0)
		v1 = (((self.calibration_p[2] * (((v1 / 4.0) * (v1 / 4.0)) / 8192)) / 8) + ((self.calibration_p[1] * v1) / 2.0)) / 262144
		v1 = ((32768 + v1) * self.calibration_p[0]) / 32768

		if v1 == 0:
			return 0

		pressure = ((1048576 - adc_p) - (v2 / 4096)) * 3125
		if pressure < 0x80000000:
			pressure = (pressure * 2.0) / v1
		else:
			pressure = (pressure / v1) * 2

		v1 = (self.calibration_p[8] * (((pressure / 8.0) * (pressure / 8.0)) / 8192.0)) / 4096
		v2 = ((pressure / 4.0) * self.calibration_p[7]) / 8192.0
		pressure += ((v1 + v2 + self.calibration_p[6]) / 16.0)
		return pressure / 100

	def compensate_temperature(self,adc_t):
		v1 = (adc_t / 16384.0 - self.calibration_t[0] / 1024.0) * self.calibration_t[1]
		v2 = (adc_t / 131072.0 - self.calibration_t[0] / 8192.0) * (adc_t / 131072.0 - self.calibration_t[0] / 8192.0) * self.calibration_t[2]
		self.t_fine = v1 + v2
		temperature = self.t_fine / 5120.0
		return temperature

	def compensate_humidity(self,adc_h):
		var_h = self.t_fine - 76800.0
		if var_h == 0:
			return 0

		var_h = (adc_h - (self.calibration_h[3] * 64.0 + self.calibration_h[4] / 16384.0 * var_h)) * (
			self.calibration_h[1] / 65536.0 * (1.0 + self.calibration_h[5] / 67108864.0 * var_h * (
				1.0 + self.calibration_h[2] / 67108864.0 * var_h)))
		var_h *= (1.0 - self.calibration_h[0] * var_h / 524288.0)

		if var_h > 100.0:
			var_h = 100.0
		elif var_h < 0.0:
			var_h = 0.0
		return var_h

	def altitude(self):
		"""L'altitude dépand de la pression au niveau de la mer actuelle"""
		pressure = self.read_pressure()[0]
		return 44330 * (1.0 - math.pow(pressure / self.sea_level_pressure, 0.1903))
        
	def setup(self):
		#print ("Calibration " + str(hex(self.Ad)) + "  " + str(self.cpt))
		osrs_t = 1	# Temperature oversampling x 1
		osrs_p = 1	# Pressure oversampling x 1
		osrs_h = 1	# Humidity oversampling x 1
		mode = 3	# Normal mode
		t_sb = 5	# Tstandby 1000ms
		filter = 0	# Filter off
		spi3w_en = 0  # 3-wire SPI Disable

		ctrl_meas_reg = (osrs_t << 5) | (osrs_p << 2) | mode
		config_reg = (t_sb << 5) | (filter << 2) | spi3w_en
		ctrl_hum_reg = osrs_h

		self.bus.write_byte_data(self.Ad,0xF2, ctrl_hum_reg)
		self.bus.write_byte_data(self.Ad,0xF4, ctrl_meas_reg)
		self.bus.write_byte_data(self.Ad,0xF5, config_reg)
		self.reset_calibration()
		self.populate_calibration_data()

	def CORtemp(self,ct):
		self.cor_t=ct
